/*
 * Turbo Programmer Utilities, turbo-prog-utils, www.bladox.com 
 *
 * Copyright (C) 2004 BLADOX, s.r.o.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>

#include <tprog/tproglib.h>
#include "../module-src/turboadapter.h"

void help ()
{
  printf ("Print Turbo Adapter debug\n");
  printf ("Usage: btldr [args] file\n");
  printf ("-a application\tuse application, default is %s\n",
	  APP_TURBO_ADAPTER);
  printf ("-d device\tuse USB device, default is %s\n", USBDEVICE);
  printf ("-h\t\thelp\n");
  printf ("-v\t\tverbose\n");
  exit (-1);
}

int main (int argc, char **argv)
{
  int result;
  u8 *dev = USBDEVICE;
  u8 *app_name = APP_TURBO_ADAPTER;
  u8 answer[2048];
  u16 answerLen;
  u32 i, i2;
  u16 l;
  u16 flashPageNumber;
  int fdstdin;
  int flags;
  int bytes_read;
  char buf[1024];
  int len;
  FILE *fe;
  u8 *filename = NULL;
  u8 c;
  u8 buf2[0x1E000];

  do
  {
    result = getopt (argc, argv, "hd:a:v");
    switch (result)
    {
      case 'a':
	app_name = optarg;
	break;
      case 'd':
	dev = optarg;
	break;
      case 'v':
	verbose_inc ();
	break;
      case 'h':
	help ();
	break;
    }
  }
  while (result > 0);

  if (argc - optind != 1)
    help ();

  filename = argv[optind];

  fe = fopen (filename, "r");
  if (fe == NULL)
  {
    printf ("Cannot open %s for writing\n", filename);
    exit (-1);
  }

  for (i2 = 0; i2 < 0x1E000; i2++)
    buf2[i2] = 0xFF;

  i2 = 0;
  while (!feof (fe))
  {
    buf2[i2++] = fgetc (fe);
  }
  fclose (fe);

  open_device (dev);
  if (select_app (app_name) != NO_ERROR)
  {
    close_device ();
    fprintf (stderr, "Cannot open application %s\n", app_name);
    exit (-1);
  }

  printf ("Turbo Adapter bootloader mode initialized, %s\n", app_name);
  printf ("BOOTLOADER is erasing FLASH and EEPROM, wait for approx. 50s\n");
  usb_send (ACTION_TURBO_ADAPTER_BTLDR_INIT, 0, NULL, &answerLen, answer);
  printf ("BOOTLOADER is updating FLASH\n");
  for (i2 = 0; i2 < 0x1E000; i2 += 256)
  {
    printf ("Page %4d/(0-%4d)\r", i2 / 256, 0x1E0 - 1);
    fflush (stdout);
    flashPageNumber = i2 / 256;	//address(last page before 4kB bootloader)= 0x1DF00 bytes(0xEF00 words) (page=256B)
    answer[0] = (flashPageNumber & 0xFF);	//low byte
    answer[1] = (flashPageNumber >> 8);	//high byte
    for (i = 0; i < 256; i++)
      answer[2 + i] = buf2[i2 + i];
    usb_send (ACTION_TURBO_ADAPTER_BTLDR_WRITE_PAGE, 2 + 256, answer,
	      &answerLen, answer);
  }
  printf ("\n");

  close_device ();
}
